/*
 * main.c
 *
 * Author: chris (at) hobbyelektronik (dot) org
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "RFM01.h"
#include "EMR7370.h"

//0, 1 or 2
#define DEBUG_LEVEL 0

#define MY_METER_COUNT 2
t_emr my_meters[MY_METER_COUNT];
volatile uint8_t timestamp;

void emrdata2uart(t_emr* data) {
	uart_puts("Addr 0x");
	uart_puthex(data->address>>8);
	uart_puthex(data->address&0xFF);
	
	uart_puts(": U=");
	uart_putu((uint16_t)(data->voltage/2) + 128);
	uart_putc('.');
	uart_putc(data->voltage & 1 ? '5' : '0');
	
	uart_puts("V I=");
	
	uint16_t current = data->current;
	uart_putu(current);
	uart_puts("mA P=");
	
	uart_putu((uint16_t)(data->power/2));
	uart_putc('.');
	uart_putc(data->voltage & 1 ? '5' : '0');
	uart_puts("W E=");
	
	uart_putu(data->energy);
	uart_puts("0Wh");
	
	uart_puts(" t=");
	uart_putu(data->last_refresh);
	uart_putc('\n');
	
}




ISR(TIMER2_COMP_vect) {
	timestamp++;
}

//Init timer2 with 10ms ticks
inline void timer2_init() {
	TCCR2 = (1<<WGM21)|(1<<CS22)|(1<<CS21)|(1<<CS20);
	OCR2 = 256-22;
	TIMSK |= (1<<OCIE2);
}




int main(void)
{

	_delay_ms(100); //let Vcc settle
	uart_init();
	RFM_Init();
	timer2_init();
	
	uart_puts("Built: " __DATE__ " " __TIME__ "\n");
	
	uint8_t timer_display = 0;
	
	sei();
	
	uint8_t last_meter = 0xFF;
	
	my_meters[0].address = 0x5472;
	my_meters[0].last_refresh = 0;
	my_meters[1].address = 0x547F;
	my_meters[1].last_refresh = 0;
	
    while(1)
    {
        if(rfm_bpos == RFM_PACKETLENGTH) {
			EMR_DecodePacket();
#if DEBUG_LEVEL == 2
			uart_puts("Rx from 0x");
			uart_puthex(emr_data.address>>8);
			uart_puthex(emr_data.address&0xFF);
			uart_putc('\n');
#elif DEBUG_LEVEL == 3
			emrdata2uart(&emr_data);
#endif
			
			for(int x = 0; x < MY_METER_COUNT; x++) {
				//update the data when matching to the given address and the last update is older than a second
				if(my_meters[x].address == emr_data.address && my_meters[x].last_refresh > 1) {
					last_meter = x;
					my_meters[x].power = emr_data.power;
					my_meters[x].voltage = emr_data.voltage;
					my_meters[x].current = emr_data.current;
					my_meters[x].energy = emr_data.energy;
					my_meters[x].last_refresh = 0;
					
					break;
				}
				
			}
	
			EMR_StartRx();
			
		}
#if DEBUG_LEVEL >= 1
		//Notify when new data arrived
		if(last_meter != 0xFF) {
			uart_puts("Receiver ");
			uart_putu(last_meter);
			uart_puts(" updated.\n");
			last_meter = 0xFF;
		}
#endif
		
		if(timestamp >= 100) {
			timestamp -= 100;
			timer_display++;
			
			uint8_t min_age = 0xFF;
			for(int x = 0; x < MY_METER_COUNT; x++) {
				if(my_meters[x].last_refresh < min_age) min_age = my_meters[x].last_refresh;
				if(my_meters[x].last_refresh < 255) my_meters[x].last_refresh++;
			}
			
			if(min_age > 30) {
				uart_puts("RFM received no data -> reset\n");
				RFM_Init();
			}
			
			if(timer_display >= 10) {
				timer_display = 0;
				for(int x = 0; x < MY_METER_COUNT; x++) {
					emrdata2uart(&my_meters[x]);
				}
			}
		}			
    }
}

